﻿'----------------------------------------------------------
' DS102/DS112コントローラ制御用サンプルプログラム
'
' Copyright 2023 SURUGA SEIKI Co.,Ltd. All rights reserved.
'----------------------------------------------------------

Public Class Form1

    Private AxisNo As String = "1"          '軸番号
    Private Direction As String             '駆動方向設定 (-(CCW)、+(CW))
    Private Mode As Short = 0               '駆動方法 (0:連続駆動、1:ステップ駆動、2:原点復帰)

    '''<summary>
    '''初期設定
    '''</summary>
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        'ソフトウェアバージョンを表示する
        Dim asm As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
        Dim ver As System.Version = asm.GetName().Version
        Dim softwareVersion As System.Version = New Version(ver.Major, ver.Minor, ver.Build)
        Text = "DS102/DS112コントローラ・サンプルプログラム Ver." + softwareVersion.ToString

        ComboBoxOrg.SelectedIndex = 0
        ComboBoxCommPort.SelectedIndex = 0
        ComboBoxBaudrate.SelectedIndex = 0
        TextBoxPosition.Text = "0"

    End Sub

    '''<summary>
    '''接続ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonConnect_Click(sender As Object, e As EventArgs) Handles ButtonConnect.Click

        '通信ポートを設定する
        CommPortOpen()

    End Sub

    '''<summary>
    '''接続ボタンを押した時の処理
    '''</summary>
    Private Sub CommPortOpen()

        If SerialPort.IsOpen = True Then
            SerialPort.Close()
        End If

        SerialPort.PortName = ComboBoxCommPort.Text                     'COM番号
        SerialPort.BaudRate = Integer.Parse(ComboBoxBaudrate.Text)      'ボーレート
        SerialPort.NewLine = vbCr                                       'デリミタコード
        SerialPort.ReadTimeout = 2000                                   '読取りタイムアウト時間
        SerialPort.WriteTimeout = 2000                                  '書込みタイムアウト時間

        Try
            '通信ポート接続
            SerialPort.Open()

        Catch ex As System.Exception

            LabelState.Text = "接続エラー発生"
            MessageBox.Show(ex.Message)
            Exit Sub

        End Try

        System.Threading.Thread.Sleep(100)

        '---------------------------------------------------------
        ' ID要求 DS102/DS112コントローラ接続確認
        '---------------------------------------------------------
        Dim data As String

        data = SendReceiveString("*IDN?")

        Dim ret As Boolean = If(Not String.IsNullOrEmpty(data), data.Contains("SURUGA,DS1"), False)

        If ret Then

            '---------------------------------------------------------
            'バージョン要求
            '---------------------------------------------------------
            data = SendReceiveString("DS102VER?")
            LabelFirmware.Text = data

            '---------------------------------------------------------
            '制御軸数ステータス
            '---------------------------------------------------------
            data = SendReceiveString("CONTA?")

            ButtonAxisX.Enabled = False
            ButtonAxisY.Enabled = False
            ButtonAxisZ.Enabled = False
            ButtonAxisU.Enabled = False
            ButtonAxisV.Enabled = False
            ButtonAxisW.Enabled = False

            If Integer.Parse(data) = 2 Then
                '取得した軸数 2(X、Y)
                '---------------------------------------------------------
                ' 表示単位設定・速度テーブル設定
                '---------------------------------------------------------
                For axNo As Integer = 1 To 2

                    SendString("AXI" + axNo.ToString + ":UNIT 0:SELSP 0")
                    System.Threading.Thread.Sleep(100)
                Next

                ButtonAxisX.Enabled = True
                ButtonAxisY.Enabled = True

            ElseIf Integer.Parse(data) = 4 Then
                '取得した軸数 4(X～U)
                '---------------------------------------------------------
                ' 表示単位設定・速度テーブル設定
                '---------------------------------------------------------
                For axNo As Integer = 1 To 4

                    SendString("AXI" + axNo.ToString + ":UNIT 0:SELSP 0")
                    System.Threading.Thread.Sleep(100)
                Next

                ButtonAxisX.Enabled = True
                ButtonAxisY.Enabled = True
                ButtonAxisZ.Enabled = True
                ButtonAxisU.Enabled = True

            ElseIf Integer.Parse(data) = 6 Then
                '取得した軸数 6(X～W)
                '---------------------------------------------------------
                ' 表示単位設定・速度テーブル設定
                '---------------------------------------------------------
                For axNo As Integer = 1 To 6

                    SendString("AXI" + axNo.ToString + ":UNIT 0:SELSP 0")
                    System.Threading.Thread.Sleep(100)
                Next

                ButtonAxisX.Enabled = True
                ButtonAxisY.Enabled = True
                ButtonAxisZ.Enabled = True
                ButtonAxisU.Enabled = True
                ButtonAxisV.Enabled = True
                ButtonAxisW.Enabled = True
            End If

            'ステータスを更新する
            UpdateStatus()

        Else
            LabelState.Text = "受信エラー発生"
            MessageBox.Show(ComboBoxCommPort.Text + "にDS102/DS112コントローラが接続されていません")

        End If

    End Sub

    '''<summary>
    '''切断ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonDisconnect_Click(sender As Object, e As EventArgs) Handles ButtonDisconnect.Click

        If SerialPort.IsOpen = True Then
            SerialPort.Close()
        End If

        ButtonAxisX.Enabled = True
        ButtonAxisY.Enabled = True
        ButtonAxisZ.Enabled = True
        ButtonAxisU.Enabled = True
        ButtonAxisV.Enabled = True
        ButtonAxisW.Enabled = True
        LabelState.Text = "未接続"

    End Sub

    '''<summary>
    '''X軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisX_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisX.CheckedChanged
        AxisNo = "1"            '使用軸をX軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''Y軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisY_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisY.CheckedChanged
        AxisNo = "2"            '使用軸をY軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''Z軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisZ_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisZ.CheckedChanged
        AxisNo = "3"            '使用軸をZ軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''U軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisU_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisU.CheckedChanged
        AxisNo = "4"            '使用軸をU軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''V軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisV_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisV.CheckedChanged
        AxisNo = "5"            '使用軸をV軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''W軸選択ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonAxisW_CheckedChanged(sender As Object, e As EventArgs) Handles ButtonAxisW.CheckedChanged
        AxisNo = "6"            '使用軸をW軸に設定
        UpdateStatus()          'ステータスを更新する
    End Sub

    '''<summary>
    '''連続駆動を押した時の処理
    '''</summary>
    Private Sub RadioButtonContinueMode_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButtonContinueMode.CheckedChanged
        ButtonCCW.Text = "- (CCW)"
        ButtonCW.Text = "+ (CW)"
        Mode = 0
    End Sub

    '''<summary>
    '''ステップ駆動を押した時の処理
    '''</summary>
    Private Sub RadioButtonStepMode_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButtonStepMode.CheckedChanged
        ButtonCCW.Text = "- (CCW)"
        ButtonCW.Text = "+ (CW)"
        Mode = 1
    End Sub

    '''<summary>
    '''原点復帰を押した時の処理
    '''</summary>
    Private Sub RadioButtonOrgMode_CheckedChanged(sender As Object, e As EventArgs) Handles RadioButtonOrgMode.CheckedChanged
        ButtonCCW.Text = "原点復帰開始"
        ButtonCW.Text = "原点復帰開始"
        Mode = 2
    End Sub

    '''<summary>
    '''停止ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonStop_Click(sender As Object, e As EventArgs) Handles ButtonStop.Click
        '---------------------------------------------------------
        ' 停止
        '---------------------------------------------------------
        SendString("STOP 0")
    End Sub

    '''<summary>
    '''CCWボタンを押した時の処理
    '''</summary>
    Private Sub ButtonCCW_MouseDown(sender As Object, e As MouseEventArgs) Handles ButtonCCW.MouseDown
        Direction = "CCW"               'モータの駆動方向をCCWに設定する
        MoveStage()                     '駆動を開始する
    End Sub

    '''<summary>
    '''CCWボタンを離した時の処理
    '''</summary>
    Private Sub ButtonCCW_MouseUp(sender As Object, e As MouseEventArgs) Handles ButtonCCW.MouseUp
        '駆動方法が連続駆動の場合
        If Mode = 0 Then
            '---------------------------------------------------------
            ' 停止
            '---------------------------------------------------------
            SendString("STOP 0")        '軸を停止する
        End If
    End Sub

    '''<summary>
    '''CWボタンを押した時の処理
    '''</summary>
    Private Sub ButtonCW_MouseDown(sender As Object, e As MouseEventArgs) Handles ButtonCW.MouseDown
        Direction = "CW"                'モータの駆動方向をCWに設定する
        MoveStage()                     '駆動を開始する
    End Sub

    '''<summary>
    '''CWボタンを離した時の処理
    '''</summary>
    Private Sub ButtonCW_MouseUp(sender As Object, e As MouseEventArgs) Handles ButtonCW.MouseUp
        '駆動方法が連続駆動の場合
        If Mode = 0 Then
            '---------------------------------------------------------
            ' 停止
            '---------------------------------------------------------
            SendString("STOP 0")        '軸を停止する
        End If
    End Sub

    '''<summary>
    '''ステージを駆動する
    '''</summary>
    Public Sub MoveStage()

        '駆動方法
        Select Case Mode
            Case 0
                '連続駆動
                'モータの駆動方向がCWの場合
                If Direction = "CW" Then
                    '---------------------------------------------------------
                    ' 連続駆動 CW方向
                    '---------------------------------------------------------
                    SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                               ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                               ":F0 " + TextBoxSpeed.Text + ":GO CWJ")
                    'モータの駆動方向がCCWの場合
                Else
                    '---------------------------------------------------------
                    ' 連続駆動 CCW方向
                    '---------------------------------------------------------
                    SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                               ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                               ":F0 " + TextBoxSpeed.Text + ":GO CCWJ")
                End If
            Case 1
                'ステップ駆動
                '---------------------------------------------------------
                ' ステップ駆動
                '---------------------------------------------------------
                SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                           ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                           ":F0 " + TextBoxSpeed.Text + ":PULS " + TextBoxStep.Text + ":GO " + Direction)
            Case 2
                '原点復帰
                '原点復帰方式を取得する
                Dim orgMode As String = ComboBoxOrg.SelectedIndex.ToString()

                '原点復帰方式を変更する
                '---------------------------------------------------------
                ' メモリスイッチ0設定
                '---------------------------------------------------------
                SendString("AXI" + AxisNo + ":MEMSW0 " + orgMode)

                System.Threading.Thread.Sleep(100)

                '---------------------------------------------------------
                ' 原点復帰
                '---------------------------------------------------------
                SendString("AXI" + AxisNo + ":L0 " + TextBoxLSpeed.Text +
                           ":R0 " + TextBoxRate.Text + ":S0 " + TextBoxSRate.Text +
                           ":F0 " + TextBoxSpeed.Text + ":GO ORG")
        End Select

        'ステータスを更新する
        Timer.Start()

    End Sub

    '''<summary>
    '''画面更新処理
    '''</summary>
    Private Sub Timer_Tick(sender As Object, e As EventArgs) Handles Timer.Tick

        If SerialPort.IsOpen = True Then
            'ステータスを更新する
            UpdateStatus()
        End If

    End Sub

    '''<summary>
    '''ステータスを更新する
    '''</summary>
    Public Sub UpdateStatus()

        Dim data As String
        Dim result As Integer

        '---------------------------------------------------------
        'ステータス3要求
        '---------------------------------------------------------
        data = SendReceiveString("AXI" + AxisNo + ":SB3?")

        '数値に変換できなければ終了する
        If Not Integer.TryParse(data, result) Then
            Timer.Stop()
            Exit Sub
        End If

        '軸選択不可の場合
        If (Integer.Parse(data) And &H1) <> &H1 Then
            LabelState.Text = "軸選択不可能"
            Timer.Stop()
        Else
            '---------------------------------------------------------
            'ステータス1要求
            '---------------------------------------------------------
            data = SendReceiveString("AXI" + AxisNo + ":SB1?")

            If (Integer.Parse(data) And &H40) = &H40 Then
                LabelState.Text = "動作中"

            ElseIf (Integer.Parse(data) And &H10) = &H10 Then
                LabelState.Text = "原点検出"
                Timer.Stop()

            ElseIf (Integer.Parse(data) And &H2) = &H2 Or (Integer.Parse(data) And &H4) = &H4 Then
                'リミット検出
                '---------------------------------------------------------
                'ステータス2要求(リミットステータス確認)
                '---------------------------------------------------------
                data = SendReceiveString("AXI" + AxisNo + ":SB2?")

                If (Integer.Parse(data) And &H3) = &H3 Then
                    LabelState.Text = "ステージ未接続"

                ElseIf (Integer.Parse(data) And &H1) = &H1 Then
                    LabelState.Text = "CWリミット検出"

                ElseIf (Integer.Parse(data) And &H2) = &H2 Then
                    LabelState.Text = "CCWリミット検出"

                ElseIf (Integer.Parse(data) And &H4) = &H4 Then
                    LabelState.Text = "CWソフトリミット検出"

                ElseIf (Integer.Parse(data) And &H8) = &H8 Then
                    LabelState.Text = "CCWソフトリミット検出"
                End If

                Timer.Stop()

            Else

                LabelState.Text = "停止"
                Timer.Stop()

            End If

            '---------------------------------------------------------
            '現在位置要求
            '---------------------------------------------------------
            data = SendReceiveString("AXI" + AxisNo + ":POS?")
            TextBoxPosition.Text = data

        End If

    End Sub

    '''<summary>
    '''送信
    '''</summary>
    '''<param name="cmd">送信文字列</param>
    Public Sub SendString(cmd As String)

        Try
            If SerialPort.IsOpen Then
                '送信
                SerialPort.WriteLine(cmd)
            End If

        Catch ex As System.Exception
            LabelState.Text = "送信エラー発生"
            MessageBox.Show(ex.Message)

        End Try

    End Sub

    '''<summary>
    '''受信
    '''</summary>
    '''<returns>受信文字列</returns>
    Public Function ReceiveString() As String

        Dim timeoutTime As Date = Date.Now.AddMilliseconds(2000)        'タイムアウト時間
        Dim rcvData As String = Nothing                                 '受信データ

        '終端文字まで受信処理を行う
        While True

            Try

                Dim len As Integer = SerialPort.BytesToRead

                'タイムアウトまで受信を試みる
                While len < 1

                    System.Threading.Thread.Sleep(50)

                    If Date.Now > timeoutTime Then
                        LabelState.Text = "通信タイムアウト発生"
                        Return Nothing
                    End If

                    len = SerialPort.BytesToRead

                End While

                Dim readBytes As Integer = 0
                Dim inByte As Byte() = New Byte(len) {}

                '受信バッファ内データ読出
                SerialPort.Read(inByte, readBytes, len)
                '受信データを文字列に格納
                Dim rcvDataTemp As String = System.Text.Encoding.ASCII.GetString(inByte)

                rcvData += rcvDataTemp

                '終端文字列の位置確認
                Dim index As Integer = rcvData.IndexOf(SerialPort.NewLine)

                '終端文字列が有ったとき
                If index > -1 Then
                    '終端文字列の削除
                    Dim rcvDataReturn As String = rcvData.Remove(index)
                    Return rcvDataReturn
                End If

            Catch ex As System.Exception

                LabelState.Text = "受信エラー発生"
                MessageBox.Show(ex.Message)
                Return Nothing

            End Try

        End While

        Return Nothing

    End Function

    '''<summary>
    '''送受信
    '''</summary>
    '''<param name="cmd">送信文字列</param>
    '''<returns>受信文字列</returns>
    Public Function SendReceiveString(cmd As String) As String

        Dim data As String = Nothing

        If SerialPort.IsOpen = True Then

            '送信
            SendString(cmd)

            System.Threading.Thread.Sleep(100)

            '受信
            data = ReceiveString()
        End If

        Return data

    End Function

    '''<summary>
    '''ポジション設定ボタンを押した時の処理
    '''</summary>
    Private Sub ButtonSetPosition_Click(sender As Object, e As EventArgs) Handles ButtonSetPosition.Click
        '---------------------------------------------------------
        '現在位置設定
        '---------------------------------------------------------
        SendString("AXI" + AxisNo + ":POS " + TextBoxPosition.Text)
    End Sub

    '''<summary>
    '''閉じるボタンを押した時の処理
    '''</summary>
    Private Sub ButtonClose_Click(sender As Object, e As EventArgs) Handles ButtonClose.Click
        Close()
    End Sub

    '''<summary>
    '''終了時の処理
    '''</summary>
    Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        If SerialPort.IsOpen = True Then
            SerialPort.Close()
        End If
    End Sub

End Class
